$global:HardwareIDs = @("*PID_07DA*", "*PID_07DC*", "*PID_0A2A*", "*PID_0A2B*","*PID_0A5A*", "*PID_0A5B*")
$global:WiFiHardwareIDs = @("*VEN_8086&DEV_08B1*", "*VEN_8086&DEV_08B2*", "*VEN_8086&DEV_08B3*", "*VEN_8086&DEV_08B4*", "*VEN_8086&DEV_095A*", "*VEN_8086&DEV_095B*")
$global:EhciControllers = @("*VEN_8086&DEV_1E26*","*VEN_8086&DEV_9C26*")
$global:XhciControllers = @("*VEN_8086&DEV_9C31*")

$global:CmdTraceValues = @("boot", "patch", "patchlib", "rfkill", "audio", "moto", "msbt", "usbbl", "profiles", "msusb")
$global:DefaultTraces = @("patch", "moto")
$global:RfkillTraces = @("patch", "rfkill")
$global:BootTraces = @("boot", "patch","moto", "msusb")
$global:SleepTraces = @("sleep","patch","moto","msusb")
$global:HibernateTraces = @("hibernate", "patch","moto","msusb")
$global:AllTraces = @("patch", "patchlib", "rfkill", "moto", "msbt", "profiles", "msusb")

$global:IntelAdapterName = "Not Found"
$global:MotAdapterName = "Not Found"
$global:DriverVersion = "0.0.0.0"
$global:UserName = [Environment]::UserName
$global:LogFolderPath = "C:\Users\$UserName\Desktop\Intel_Bluetooth_Traces"
$global:LogFilePath = ""
$global:SessionState = "Enabled"
$global:ComputerName = (Get-Item env:\Computername).Value 
$global:DriverVersion = "0.0.0.0"
$global:MotoStackVersion = "0.0.0.0"
$global:DeviceHID = "NULL"
$global:IntelBTStack = $false
$global:MotorolaBTStack = $false
$global:IntelAudioStack = $false
$global:PatchTraceStarted = 0
$global:HostController = ""
$global:3rdParty = $false;
$global:OSVersion = ""
$global:BootTraceOn = $false
$global:XHCI = $false;
$global:EHCI = $false;
$global:USB = $false;
$global:OptionsFile = "Options.log"
$global:SummaryFile = "Summary.log"
$global:LinkKeyFile = "LinkKeys.log"
$global:DeviceId = ""
$global:HardwareId = ""
$global:DriverVersion = ""
$global:Version = "Version 1.7.10"

function GetOSVersion() 
{
    $global:OSVersion = "Win7";

    if ([Environment]::OSVersion.Version -ge (new-object 'Version' 6,1))
    {
        $global:OSVersion = "Win7"
    }

    if ([Environment]::OSVersion.Version -ge (new-object 'Version' 6,2))
    {
        $global:OSVersion = "Win8"
    }

    if ([Environment]::OSVersion.Version -ge (new-object 'Version' 6,3))
    {
        $global:OSVersion = "Win8.1"
    }
}

function ProbeHostController()
{
    foreach ($hid in $global:HardwareIds) 
    {
        $Controller = gwmi win32_usbcontrollerdevice | where-object { $_.Dependent -like $hid };

        foreach ($cid in $global:XhciControllers)
        {
            if ($Controller.Antecedent -like $cid) 
            {
                LogMessage("Host controller is XHCI");
                $global:XHCI = $true;
                break;  
            }
        }

        foreach ($cid in $global:EhciControllers)
        {
            if ($Controller.Antecedent -like $cid) 
            {
                $global:EHCI = $true;
                LogMessage("Host controller is EHCI");
                break;  
            }
        }
    }
}

function ProbeHardwareAndSoftware()
{
    $result = $false;

    write-host "Probing hardware...";

    foreach($Hid in $HardwareIDs)
    {
        $driver = gwmi win32_pnpsigneddriver -namespace root\cimv2 | Where-Object {$_.DeviceID -like $Hid};

        if ($driver)
        {
            $result = $true;

            $global:DeviceId = $driver.DeviceID;
            $global:DriverVersion = [System.Diagnostics.FileVersionInfo]::GetVersionInfo("c:\windows\system32\drivers\ibtusb.sys").FileVersion
            $global:HardwareId = $driver.HardwareID;
            $global:PnPDescription =  $driver.Description;
            
            $infFile = "c:\windows\inf\" + $driver.InfName;

            LogMessage("Found $global:PnPDescription($global:HardwareId)");
            LogMessage("Ibtusb.sys Version: $DriverVersion")

            write-host "Probing software...";

            $infContent = Get-Content $infFile -ReadCount 0
            
            if ($infContent -match 'btmhsf.sys')
            {
                $global:MotorolaBTStack = $true  
                LogMessage("Found Intel/Motorola Bluetooth stack");
                if (Test-Path c:\windows\system32\drivers\btmhsf.sys) {
                    $global:MotoStackVersion = [System.Diagnostics.FileVersionInfo]::GetVersionInfo("c:\windows\system32\drivers\btmhsf.sys").FileVersion
                    LogMessage("Motorola Stack Version: $MotoStackVersion")
                }
            }else
            {
                $global:IntelBTStack = $true
                LogMessage("Found Inbox Bluetooth Stack with Firwmare update support");
 
            }
            break;
        }
    }

    if ([string]::IsNullOrEmpty($global:DeviceId))
    {
        $global:3rdParty = $true;
        LogMessage("Found 3rd party Bluetooth module. Only selective trace options shall be supported");
    }
    
    foreach($Hid in $WiFiHardwareIDs)
    {
        $WiFidriver = gwmi win32_pnpsigneddriver -namespace root\cimv2 | Where-Object {$_.DeviceID -like $Hid};
	if ($WiFidriver) 
        {
           $WiFiDriverHWID = $WiFidriver.HardwareID
           $WiFiDriverVer = $WiFidriver.DriverVersion
           LogMessage("Found WiFi: $WiFiDriverHWID Driver Version: $WiFiDriverVer")
        }
    }
    
    ProbeHostController;

    GetOSVersion;
}

function LogMessage($text)
{
    write-host $text;
    Add-Content $global:SummaryFile $text;
}

# Check if BT is on EHCI/XHCI
function CheckHostControllerMode($HwId)
{
    $PnPDevices = gwmi -Query "select * from Win32_USBControllerDevice"

    foreach ($PnPDevice in $PnPDevices ) 
    {
        if($PnPDevice.Dependent.Contains($HardwareID))
        {
            foreach($Controller in $EhciControllers) 
            {
                if ($PnPDevice.Antecedent.Contains($Controller))
                {
                    $global:HostController = "EHCI";
                    return $global:HostController;
                }
            }

            foreach($Controller in $XhciControllers) 
            {
                if ($PnPDevice.Antecedent.Contains($Controller))
                {
                    $global:HostController = "XHCI";
                    return $global:HostController;
                }
            }
        }
    }

    return "";
}

function DumpEventLog($Type) 
{
    $FileName = "$global:LogFilePath\$Type.log";
    Get-EventLog -logname $Type -ea SilentlyContinue | ?{$_.timegenerated -gt [datetime]::Today} | Format-List | Out-File $FileName;
}

function StartMsUsbLogging()
{
    if ($global:BootTraceOn)
    {
        if ($global:XHCI -eq $true)
        {
            logman create trace autosession\msusb -o "$LogFilePath\msusb-xhci.etl" -pf .\providers\msft\msusb3.guid -ets -nb 128 512 -mode append -ct system | Out-Null
        }

        if ($global:EHCI -eq $true)
        {
             logman create trace autosession\msusb -o "$LogFilePath\msusb-ehci.etl" -pf .\providers\msft\msusb2.guid -ets -nb 128 512 -mode append -ct system | Out-Null
        }
    }else
    {
        if ($global:EHCI -eq $true)
        {
            logman create trace msusb -o "$LogFilePath\msusb-ehci-%d.etl"  -pf .\providers\msft\msusb2.guid -mode newfile -max 32 -nb 128 512 -ets | Out-Null
        }

        if ($global:XHCI -eq $true)
        {
          logman create trace msusb -o "$LogFilePath\msusb-xhci-%d.etl"  -pf .\providers\msft\msusb3.guid -mode newfile -max 32 -nb 128 512 -ets | Out-Null
        }
    }
}

function StopMsUsbLogging()
{
    logman stop msusb -ets | Out-Null
     
    if ($global:BootTraceOn)
    {
        logman delete autosession\msusb -ets | Out-Null
    }else
    {
        logman delete msusb  | Out-Null
    }
}

function StartSivaLogging()
{
    if ($global:3rdParty -eq $true)
    {
        return;
    }

    logman create trace siva -o "$LogFilePath\ibtsiva.etl" -pf .\providers\intel\siva.guid -ets | Out-Null
}

function StopSivaLogging()
{
    logman stop siva -ets | Out-Null
    logman delete siva | Out-Null
}

function StartiBtUsbBootloaderLogging()
{
    logman create trace ibtusbbl -o "$LogFilePath\ibtusbbl.etl" -pf .\providers\intel\ibtusbbl.guid -nb 128 512 -bs 512 -ets | Out-Null
}

function StopiBtUsbBootloaderLogging()
{
    logman stop ibtusbbl -ets | Out-Null
}

function StartUsbDriverLogging()
{
    if ($global:3rdParty -eq $true)
    {
        return;
    }

    if ($global:BootTraceOn -eq $false)
    {
        if ($global:PatchTraceStarted -eq $false) 
        {
            logman create trace ibt -o "$LogFilePath\ibt-$DriverVersion-%d.etl" -pf .\providers\intel\ibt-usb.guid -mode newfile -nb 128 512 -bs 512 -max 32 -ets | Out-Null
            $global:PatchTraceStarted = $true;
        }
    }else
    {
         logman create trace autosession\ibt -o "$LogFilePath\ibt-$DriverVersion.etl" -pf .\providers\intel\ibt-usb.guid -ets -nb 128 512 -bs 512 --a -ct system | Out-Null
         $global:PatchTraceStarted = $true;
    }
}

function StopUsbDriverLogging()
{
    logman stop ibt -ets | Out-Null
     
    if ($global:BootTraceOn)
    {
        logman delete autosession\ibt -ets | Out-Null
    }else
    {
        logman delete ibt  | Out-Null
    }
}

function StartUartDriverLogging()
{
    if ($global:3rdParty -eq $true)
    {
        return;
    }
}

function StopUartDriverLogging()
{
}

function StartPatchLibLogging()
{
    if ($global:3rdParty -eq $true)
    {
        return;
    }

    if ($global:PatchTraceStarted -eq $false) 
    {
        logman create trace ibt  -pf .\providers\intel\patchlib.guid -o "$LogFilePath\ibt-$DriverVersion.etl" -nb 128 512 -bs 512 -ets | Out-Null
        $global:PatchTraceStarted = $true;
    } else 
    {
        logman update trace ibt -pf .\providers\intel\patchlib.guid -ets | Out-Null
    }
}

function StopPatchLibLogging()
{
    logman stop ibt -ets  | Out-Null
    logman delete ibt | Out-Null
}

function StartRfKillLogging () 
{
    if ($global:3rdParty -eq $true)
    {
        return;
    }

    if ($global:OSVersion -eq "Win8")
    {
        logman create trace ibt3-rfkill -o "$LogFilePath\ibt3-rfkill-$DriverVersion.etl" -pf .\providers\intel\rfkill.guid -ets  -nb 128 512 -bs 512 | Out-Null
    }
}

function StopRfKillLogging()
{
    logman stop ibt3-rfkill -ets | Out-Null
    logman delete ibt3-rfkill | Out-Null
}

function StartBTAudioLogging
{
    if ($global:3rdParty -eq $true)
    {
        return;
    }

    logman create trace ibt3-audio -o "$LogFilePath\ibt3audio-$DriverVersion.etl" -p "{957ba0b2-00b8-4ab1-adb8-ea812b957ce2}" 0xFFFFFFFF 0xFF -ets -nb 128 512 -bs 512 | Out-Null
}

function StopBTAudioLogging()
{
    logman stop ibt3-audio -ets | Out-Null
    logman delete ibt3-audio | Out-Null
}

function StartMsBtLogging()
{
    if ($global:BootTraceOn -eq $false)
    {
        logman create trace msbt -o "$LogFilePath\msbt-%d.etl" -p Microsoft-Windows-BTH-BTHUSB -ets -nb 128 512 -bs 512 -mode newfile -max 32 
        logman update trace msbt -p Microsoft-Windows-BTH-BTHPORT  -ets | Out-Null
    }else
    {
        .\tools\tracelog.exe -addautologger msbt -sessionguid "#{8BC8C884-DE76-4A9B-8817-6D0B6CCBAD3C}" -b 64 -ft 1 -guid ".\providers\msft\msbt.guid" -f "$LogFilePath\msbt.etl"  -flag 0xFFFFFF -level 0xFF
    }
}

function StopMsBtLogging()
{
    if ($global:BootTraceOn -eq $false)
    {
        logman stop msbt -ets | Out-Null
        logman delete msbt -ets | Out-Null
    }else
    {
        .\tools\tracelog -stop msbt | Out-Null
        .\tools\tracelog -remove msbt | Out-Null
    }
}

function StartMotorolaBootLogging()
{
    if ($global:3rdParty -eq $true)
    {
        return;
    }

    if ($global:MotorolaBTStack -eq $false)
    {
        return;
    }

    .\tools\tracelog.exe -addautologger BTMPP_BOOT_Trace -sessionguid "#{3736FF88-3900-4dee-BEBC-10256F2560AF}" -b 1024 -ft 1 -guid ".\motorola\providers\providers-common.guid" -f "$LogFilePath\mot-log-boot.etl" -level 0xFF -matchanykw 0xFFFFFFFF -level 0xFF -usesystemtime -max 52 -append | Out-Null
}

function StopMotorolaBootLogging()
{
    .\tools\tracelog.exe -stop BTMPP_BOOT_Trace | Out-Null
    .\tools\tracelog.exe -remove BTMPP_BOOT_Trace  | Out-Null
}

function StartMotorolaCommonLogging() 
{   
    if ($global:3rdParty -eq $true)
    {
        return;
    }

    if ($global:MotorolaBTStack -eq $false)
    {
        return;
    }

    $env:KM_SESSION_NAME='BTMPP_Common_KM_Trace';
    $env:UM_SESSION_NAME='BTMPP_Common_UM_Trace';
    $env:DRV_PROVIDERS=".\providers\motorola\providers-drivers.guid"
    $env:APP_PROVIDERS=".\providers\motorola\providers-app.guid"

    .\tools\tracelog.exe -stop $env:UM_SESSION_NAME  | Out-Null
    .\tools\tracelog.exe -stop $env:KM_SESSION_NAME  | Out-Null

    .\tools\tracelog.exe -start $env:UM_SESSION_NAME -guid $env:APP_PROVIDERS -f "$LogFilePath\log_um_common_%d.etl" -newfile 1000 -b 1024 -ft 2 -min 64 | Out-Null
    .\tools\tracelog.exe -start $env:KM_SESSION_NAME -guid $env:DRV_PROVIDERS -f "$LogFilePath\log_km_common_%d.etl" -newfile 1000 -b 1024 -ft 2 -min 64 | Out-Null
}

function StopMotorolaCommonLogging()
{
    $env:KM_SESSION_NAME='BTMPP_Common_KM_Trace';
    $env:UM_SESSION_NAME='BTMPP_Common_UM_Trace';

    .\tools\tracelog.exe -stop $env:UM_SESSION_NAME  | Out-Null
    .\tools\tracelog.exe -stop $env:KM_SESSION_NAME | Out-Null
}

function StartMotorolaLeHidLogging
{
    if ($global:3rdParty -eq $true)
    {
        return;
    }

    if ($global:MotorolaBTStack -eq $false)
    {
        return;
    }

    $env:KM_SESSION_NAME='BTMPP_LEHID_KM_Trace';
    $env:UM_SESSION_NAME='BTMPP_LEHID_UM_Trace';
    $env:DRV_PROVIDERS=".\providers\motorola\providers-lehidk.guid"
    $env:APP_PROVIDERS=".\providers\motorola\providers-lehidu.guid"

    .\tools\tracelog.exe -stop $env:UM_SESSION_NAME  | Out-Null
    .\tools\tracelog.exe -stop $env:KM_SESSION_NAME  | Out-Null

    .\tools\tracelog.exe -start $env:UM_SESSION_NAME -guid $env:APP_PROVIDERS -f "$LogFilePath\log_um_lehid_%d.etl" -newfile 1000 -b 1024 -ft 2 -min 64 | Out-Null
    .\tools\tracelog.exe -start $env:KM_SESSION_NAME -guid $env:DRV_PROVIDERS -f "$LogFilePath\log_km_lehid_%d.etl" -newfile 1000 -b 1024 -ft 2 -min 64 | Out-Null
}

function StopMotorolaLeHidLogging
{
    $env:KM_SESSION_NAME='BTMPP_LEHID_KM_Trace';
    $env:UM_SESSION_NAME='BTMPP_LEHID_UM_Trace';

    .\tools\tracelog.exe -stop $env:UM_SESSION_NAME | Out-Null
    .\tools\tracelog.exe -stop $env:KM_SESSION_NAME | Out-Null
}

function StartMotorolaMediaLogging
{
    if ($global:3rdParty -eq $true)
    {
        return;
    }

    if ($global:MotorolaBTStack -eq $false)
    {
        return;
    }

    $env:KM_SESSION_NAME='BTMPP_Media_KM_Trace'
    $env:UM_SESSION_NAME='BTMPP_Media_UM_Trace'
    $env:DRV_PROVIDERS=".\providers\motorola\providers-drivers.guid"
    $env:APP_PROVIDERS=".\providers\motorola\providers-media.guid"

    .\tools\tracelog.exe -stop $env:UM_SESSION_NAME | Out-Null
    .\tools\tracelog.exe -stop $env:KM_SESSION_NAME | Out-Null

    .\tools\tracelog.exe -start $env:UM_SESSION_NAME -guid $env:APP_PROVIDERS -f "$LogFilePath\log_um_media_%d.etl" -newfile 10000 -b 1024 -ft 2 -min 64 | Out-Null
    .\tools\tracelog.exe -start $env:KM_SESSION_NAME -guid $env:DRV_PROVIDERS -f "$LogFilePath\log_km_media_%d.etl" -newfile 10000 -b 1024 -ft 2 -min 64 | Out-Null
}

function StopMotorolaMediaLogging
{
    $env:KM_SESSION_NAME='BTMPP_Media_KM_Trace'
    $env:UM_SESSION_NAME='BTMPP_Media_UM_Trace'

    .\tools\tracelog.exe -stop $env:UM_SESSION_NAME | Out-Null
    .\tools\tracelog.exe -stop $env:KM_SESSION_NAME | Out-Null
}

function StartMotorolaObexLogging
{
    if ($global:3rdParty -eq $true)
    {
        return;
    }

    if ($global:MotorolaBTStack -eq $false)
    {
        return;
    }

    $env:KM_SESSION_NAME='BTMPP_OBEX_KM_Trace'
    $env:UM_SESSION_NAME='BTMPP_OBEX_UM_Trace'
    $env:DRV_PROVIDERS=".\providers\motorola\providers-drivers.guid"
    $env:APP_PROVIDERS=".\providers\motorola\providers-obex.guid"

    .\tools\tracelog.exe -stop $env:UM_SESSION_NAME | Out-Null
    .\tools\tracelog.exe -stop $env:KM_SESSION_NAME | Out-Null

    .\tools\tracelog.exe -start $env:UM_SESSION_NAME -guid $env:APP_PROVIDERS -f "$LogFilePath\log_um_obex_%d.etl" -newfile 10000 -b 1024 -ft 2 -min 64 | Out-Null
    .\tools\tracelog.exe -start $env:KM_SESSION_NAME -guid $env:DRV_PROVIDERS -f "$LogFilePath\log_km_obex_%d.etl" -newfile 10000 -b 1024 -ft 2 -min 64 | Out-Null
}

function StopMotorolaObexLogging()
{
    $env:KM_SESSION_NAME='BTMPP_OBEX_KM_Trace'
    $env:UM_SESSION_NAME='BTMPP_OBEX_UM_Trace'

    .\tools\tracelog.exe -stop $env:UM_SESSION_NAME | Out-Null
    .\tools\tracelog.exe -stop $env:KM_SESSION_NAME | Out-Null
}

function BTLogSystemInfo() 
{
    Systeminfo > "$LogFilePath\systeminfo.txt";
}

function BTLogTraceToolVersionInfo()
{
   $global:version > "$LogFilePath\Summary.txt";
}

function BTGetInstalledSoftwareInfo() 
{
    $Softwares = gwmi -class win32_product | Where-Object {$_.Vendor -like "*Intel*"}

    echo $Software >> "$LogFilePath\SoftwareInfo.txt";
}

function CreateSessionFolder($UserPath) 
{    
    $LocalPath = "null"

    $Date = Get-Date;
    $Hour = $Date.Hour;
    $Minute = $Date.Minute;
    $Second = $Date.Second;
    $Month = $Date.Month;
    $Day = $Date.Day;

    $LogFolder = "$env:COMPUTERNAME-$Month-$Day-$Hour$Minute$Second";

    $LocalPath = "$UserPath\$LogFolder";
        
    ### Create the directory
    New-Item -type Directory -path $LocalPath

    return [string]$LocalPath.trim();
}

function ProbeLinkKeys()
{
    ### Access to HKLM:\SYSTEM\CurrentControlSet\services\BTHPORT\Parameters\Keys is restricted to LocalSystem account.
    ### Check for presence of psexec tool, and attempt to gather link keys
    $BtNetworkAdapters = Get-wmiobject win32_NetworkAdapter | Where-Object {$_.Description -like "*Bluetooth*"} | Where-Object {$_.MACAddress}
    if ($BtNetworkAdapters) 
    {
        if (Test-Path .\tools\PsExec.exe) 
        {
	       & .\tools\PsExec.exe -i -s powershell.exe -ExecutionPolicy Bypass -File "$PWD\scripts\GetLinkKeys.ps1" -myparam "$LogFilePath\$LinkKeyFile"
        }
    }
}

function Probeibtver()
{
    if (Test-Path .\tools\ibtver.exe)
    {
        $temp = .\tools\ibtver.exe
        LogMessage("$temp")
    }
}